import os as Os;
import ssl as Ssl;
import copy as Copy;
import time as Time;
import uuid as Uuid;
import orjson as Json;
import socket as Socket;
import base64 as Base64;
import struct as Struct;
import psutil as Psutil;
import shutil as Shutil;
import hashlib as Hashlib;
import asyncio as Asyncio;
import platform as Platform;
import functools as Functools;
import traceback as Traceback;
import subprocess as Subprocess;
from urllib import parse as Parse;
import multiprocessing as Processing;
from collections import deque as Deque;
from types import SimpleNamespace as Simple;
from http.cookies import SimpleCookie as Cookie;
from setproctitle import setproctitle as Proctitle;
from datetime import datetime as Date , timedelta as Delta;
from concurrent.futures import ThreadPoolExecutor as Executor;
try : from uvloop import EventLoopPolicy;
except : pass;


from . import Library;
from .Function import Function;

Proctitle = 'Overspeed';

Http_Status = True;
Http_Print = False;
Http_Journal = True;
Http_Cpu = Library.Os.cpu_count();
Http_Port = [31100];
Http_Large = 2097152;
Http_Keyfile = str();
Http_Certfile = str();
Http_Callable = None;

Websocket_Status = True;
Websocket_Print = False;
Websocket_Journal = True;
Websocket_Cpu = Library.Os.cpu_count();
Websocket_Port = [31200];
Websocket_Timeout = 5;
Websocket_Connect = 60;
Websocket_Keyfile = str();
Websocket_Certfile = str();
Websocket_Callable = None;

Resource_Status = True;
Resource_Print = False;
Resource_Journal = True;
Resource_Sleep = 30;
Resource_Cpu = True;
Resource_Memory = True;
Resource_Network = True;
Resource_Disk = True;
Resource_File = True;
Resource_Load = True;

Journal_Thread = 100;
Journal_Clean = 7;

Runtime_Trace = '/Runtime/Trace/';
Runtime_Http = '/Runtime/Http/';
Runtime_Websocket = '/Runtime/Websocket/';
Runtime_Resource = '/Runtime/Resource/';

Socket_Host = '0.0.0.0';
Socket_Waittime = 5;
Socket_Listen = int(Function.Sysctl('net.core.somaxconn' , 1024));
Socket_Empty = b'';
Socket_Newline = b'\r\n';
Socket_Keep = b'keep-alive';
Socket_Close = b'close';
Socket_Cors = Socket_Newline.join([
    b'Access-Control-Allow-Origin:*' ,
    b'Access-Control-Allow-Methods:GET,POST,PUT,DELETE,OPTIONS' ,
    b'Access-Control-Allow-Headers:*' ,
    Socket_Empty ,
]);
Socket_Response = Socket_Newline.join([
    b'HTTP/1.1 %s' ,
    b'Content-Type:%s' ,
    b'Content-Length:%d' ,
    b'Connection:%s' ,
    b'%s' ,
    b'%s' ,
]);
Socket_Upgrade = Socket_Newline.join([
    b'HTTP/1.1 101 Switching Protocols' ,
    b'Upgrade:websocket' ,
    b'Connection:Upgrade' ,
    b'Sec-WebSocket-Accept:%s' ,
    Socket_Empty ,
    Socket_Empty ,
]);
Socket_Guid = '258EAFA5-E914-47DA-95CA-C5AB0DC85B11';
Socket_Fin = 0b10000000;
Socket_Opcode = 0b00001111;
Socket_Payload = 0b01111111;
Socket_H = '>H';
Socket_Q = '>Q';

Socket_Html = b'text/html';
Socket_Image = b'image/*';
Socket_Plain = b'text/plain';
Socket_Xml = b'application/xml';
Socket_Json = b'application/json';
Socket_Stream = b'text/event-stream';
Socket_Data = b'multipart/form-data';
Socket_Javascript = b'application/javascript';
Socket_Url = b'application/x-www-form-urlencoded';

Socket_200 = b'200 OK';
Socket_204 = b'204 No Content';
Socket_301 = b'301 Moved Permanently';
Socket_302 = b'302 Found';
Socket_400 = b'400 Bad Request';
Socket_401 = b'401 Unauthorized';
Socket_402 = b'402 Bad Internal';
Socket_403 = b'403 Forbidden';
Socket_404 = b'404 Not Found';
Socket_405 = b'405 Method Not Allowed';
Socket_406 = b'406 Not Acceptable';
Socket_407 = b'407 Body Error';
Socket_408 = b'408 Request Timeout';
Socket_413 = b'413 Payload Too Large';
Socket_415 = b'415 Unsupported Media Type';
Socket_429 = b'429 Too Many Requests';
Socket_500 = b'500 Internal Server Error';
Socket_502 = b'502 Bad Gateway';
Socket_503 = b'503 Service Unavailable';
Socket_504 = b'504 Gateway Timeout';


from . import Library , Const;
from .Function import Function;
from .Control import Control;

class Http :
    
    @classmethod
    def Lunch(Self , Context:Library.Simple) -> None :
        Context.Signal = Library.Simple(
            Proctitle = f'{Context.Config.Proctitle}.Http' ,
            Config = Context.Config.Http ,
            Handle = Self.Client ,
        );
        Control.Lunch(Context);
    
    @classmethod
    async def Client(Self , Context:Library.Simple , Reader:Library.Asyncio.StreamReader , Writer:Library.Asyncio.StreamWriter) -> None :
        try :
            Queue = Library.Asyncio.Queue();
            Waiting = Library.Asyncio.create_task(Self.Worker(Queue , Context.Journal , Writer));
            while True :
                Context = await Control.Request(Context , Reader);
                if Context.Signal.Request.Status is False : break;
                if Context.Signal.Config.Journal is True : Context.Journal.put(Library.Simple(
                    Method = 'Http' ,
                    Type = 'Request' ,
                    Body = Library.Copy.deepcopy(Context.Signal) ,
                ));
                if len(Context.Signal.Request.Header) == 0 : raise ValueError(402);
                if Context.Signal.Request.Length > Context.Signal.Config.Large : raise ValueError(413);
                if Context.Signal.Request.Method == 'OPTIONS' : raise ValueError(204);
                await Queue.put(Library.Copy.deepcopy(Context.Signal));
            await Queue.put(None);
            await Waiting;
        except ValueError as Error :
            try :
                Context.Signal.Response = dict(Code = Error.args[0]);
                await Self.Response(Context , Writer);
            except : Function.Trace(Context.Journal);
        except : Function.Trace(Context.Journal);
        finally : await Library.Asyncio.sleep(0);
    
    @classmethod
    async def Worker(Self , Queue:Library.Asyncio.Queue , Journal:Library.Processing.Queue , Writer:Library.Asyncio.StreamWriter) -> None :
        while True :
            Context = await Queue.get();
            if Context is None : break;
            await Self.Task(Library.Simple(
                Signal = Context ,
                Journal = Journal ,
            ) , Writer);
    
    @classmethod
    async def Task(Self , Context:Library.Simple , Writer:Library.Asyncio.StreamWriter) -> None :
        try :
            Context.Signal.Response = await Library.Asyncio.to_thread(Control.Business , Context);
            await Self.Response(Context , Writer);
        except :
            try :
                Context.Signal.Response = dict(Code = 503);
                await Self.Response(Context , Writer);
            except : pass;
            finally : Function.Trace(Context.Journal);
    
    @staticmethod
    async def Response(Context:Library.Simple , Writer:Library.Asyncio.StreamWriter) -> None :
        try :
            if Writer.is_closing() is False :
                Code = Context.Signal.Response.get('Code' , 200);
                Type = Context.Signal.Response.get('Type' , str());
                Body = Context.Signal.Response.get('Body' , str());
                Response_Code = getattr(Const , f'Socket_{Code}' , Const.Socket_400);
                if all([
                    not Code == 200 ,
                    any([
                        Body == '' ,
                        Body is None ,
                        len(Body) == 0 ,
                    ]) ,
                ]) is True : Body = Response_Code;
                if Body is None : Body = Const.Socket_Empty;
                if isinstance(Body , str) is True : Body = Body.encode();
                if any([
                    Type == '' ,
                    Type is None ,
                ]) is True :
                    Accept = Context.Signal.Request.Header.get('content-type' , Context.Signal.Request.Header.get('accept' , Const.Socket_Html));
                    if Const.Socket_Image is Accept : Response_Type = Const.Socket_Image;
                    elif Const.Socket_Plain is Accept : Response_Type = Const.Socket_Plain;
                    elif Const.Socket_Xml is Accept : Response_Type = Const.Socket_Xml;
                    elif Const.Socket_Json is Accept : Response_Type = Const.Socket_Json;
                    elif Const.Socket_Stream is Accept : Response_Type = Const.Socket_Stream;
                    elif Const.Socket_Data is Accept : Response_Type = Const.Socket_Data;
                    elif Const.Socket_Javascript is Accept : Response_Type = Const.Socket_Javascript;
                    elif Const.Socket_Url is Accept : Response_Type = Const.Socket_Url;
                    else : Response_Type = Const.Socket_Html;
                else : Response_Type = Type.encode();
                Response_Cors = Const.Socket_Cors if Context.Signal.Request.Cors is True else Const.Socket_Empty;
                Writer.write(Const.Socket_Response % (
                    Response_Code ,
                    Response_Type ,
                    len(Body) ,
                    Const.Socket_Keep ,
                    Response_Cors ,
                    Body ,
                ));
                await Writer.drain();
                if Context.Signal.Config.Journal is True : Context.Journal.put(Library.Simple(
                    Method = 'Http' ,
                    Type = 'Response' ,
                    Body = Context.Signal ,
                ));
        except : Function.Trace(Context.Journal);
        finally : await Library.Asyncio.sleep(0);


from . import Library;
from .Function import Function;
from .Http import Http;
from .Websocket import Websocket;
from .Resource import Resource;
from .Journal import Journal;

def Lunch(*Argrament , **Keyword) -> None :
    try :
        Context = Library.Simple(
            Config = Function.Configure(Keyword) ,
            Journal = Library.Processing.Queue() ,
        );
        Function.Proctitle(Context.Config.Proctitle);
        Process:list[Library.Processing.Process] = list();
        if Context.Config.Http.Status is True : Process.append(Library.Processing.Process(target = Http.Lunch , args = (Context , )));
        if Context.Config.Websocket.Status is True : Process.append(Library.Processing.Process(target = Websocket.Lunch , args = (Context , )));
        if len(Process) == 0 : Function.Print('Http Or Websocket Must Have One Enabled');
        else :
            if Context.Config.Resource.Status is True : Process.append(Library.Processing.Process(target = Resource.Lunch , args = (Context , )));
            Process.append(Library.Processing.Process(target = Journal.Lunch , args = (Context , )));
        for X in Process : X.start();
        for X in Process : X.join();
    except : Function.Trace();


from . import Library;

class Function :
    
    @staticmethod
    def Policy() -> None :
        try : Library.Asyncio.set_event_loop_policy(Library.EventLoopPolicy());
        except : pass;
    
    @staticmethod
    def Proctitle(Source:str) -> None :
        Library.Proctitle(f'{Source}.{Library.Os.getpid()}');
    
    @classmethod
    def Trace(Self , Journal = None) -> None :
        if Journal is None : Self.Print(Library.Traceback.format_exc());
        else : Journal.put(Library.Simple(
            Method = 'Trace' ,
            Body = Library.Traceback.format_exc() ,
        ));
    
    @staticmethod
    def Print(Source:str , File = None) -> None :
        print(Source , end = '\n' , file = File);
    
    @staticmethod
    def Sysctl(Primary:str , Default:str|None = None) -> str|None :
        try : return Library.Subprocess.run([
            'sysctl' ,
            '-n' ,
            Primary ,
        ] , capture_output = True , text = True , check = True).stdout.strip();
        except : return Default;
    
    @staticmethod
    def Hash(Source:str) -> str :
        if isinstance(Source , bytes) is False : Source = Source.encode();
        return Library.Base64.b64encode(Library.Hashlib.sha1(Source).digest()).decode();
    
    @classmethod
    def Ssl(Self , Source:dict) -> Library.Ssl.SSLContext|None :
        if any([
            Source.Keyfile == '' ,
            Source.Certfile == '' ,
        ]) is True : return None;
        Primary = f'SSLContext_{Self.Hash(Library.Json.dumps(dict(
            Keyfile = Source.Keyfile ,
            Certfile = Source.Certfile ,
        )))}'.upper();
        if hasattr(Self , Primary) is False :
            SSLContext = Library.Ssl.SSLContext(Library.Ssl.PROTOCOL_TLS_SERVER);
            SSLContext.load_cert_chain(
                keyfile = f'{Library.Os.getcwd()}{Source.Keyfile}' ,
                certfile = f'{Library.Os.getcwd()}{Source.Certfile}' ,
            );
            SSLContext.set_ciphers('ECDHE+AESGCM');
            SSLContext.options |= Library.Ssl.OP_NO_COMPRESSION;
            setattr(Self , Primary , SSLContext);
        return getattr(Self , Primary);
    
    @staticmethod
    def Bytes(Source:int|float , Suffix:str = 'B') -> str :
        for Unit in ['' , 'K' , 'M' , 'G' , 'T' , 'P'] :
            if abs(Source) < 1024 : return f'{Source:.2f}{Unit}{Suffix}';
            Source /= 1024;
        return f'{Source:.2f}Y{Suffix}';
    
    @staticmethod
    def Time(Method:str|bool|None = None , Utc:bool = True , Int:bool|None = None) -> str|int|float|Library.Date :
        Calendar = Library.Date.utcnow() if Utc is True else Library.Date.now();
        if isinstance(Int , bool) is True :
            Stamp = Calendar.timestamp();
            return int(Stamp) if Int is True else float(Stamp);
        elif any([
            isinstance(Method , str) is True ,
            isinstance(Method , bool) is True ,
        ]) is True :
            if Method is True : Strftime = '%Y-%m-%d %H:%M:%S';
            elif Method is False : Strftime = '%Y-%m-%d';
            else : Strftime = Method;
            return Calendar.strftime(Strftime);
        else : return Calendar;
    
    @staticmethod
    def Uuid() -> str :
        return str(Library.Uuid.uuid4()).replace('-' , '');
    
    @staticmethod
    def Configure(Keyword) -> Library.Simple :
        from . import Const;
        return Library.Simple(
            Proctitle = Keyword.get('Proctitle') if isinstance(Keyword.get('Proctitle') , str) is True and not Keyword.get('Proctitle' , '') == '' else Const.Proctitle ,
            Http = Library.Simple(
                Status = Keyword.get('Http_Status') if isinstance(Keyword.get('Http_Status') , bool) is True else Const.Http_Status ,
                Print = Keyword.get('Http_Print') if isinstance(Keyword.get('Http_Print') , bool) is True else Const.Http_Print ,
                Journal = Keyword.get('Http_Journal') if isinstance(Keyword.get('Http_Journal') , bool) is True else Const.Http_Journal ,
                Cpu = int(Keyword.get('Http_Cpu') if isinstance(Keyword.get('Http_Cpu') , int) is True and Keyword.get('Http_Cpu') > 0 else Const.Http_Cpu) ,
                Port = Keyword.get('Http_Port') if isinstance(Keyword.get('Http_Port') , list) is True and len(Keyword.get('Http_Port')) > 0 else Const.Http_Port ,
                Large = int(Keyword.get('Http_Large') if isinstance(Keyword.get('Http_Large') , int) is True and Keyword.get('Http_Large') > 0 else Const.Http_Large) ,
                Keyfile = Keyword.get('Http_Keyfile') if isinstance(Keyword.get('Http_Keyfile') , str) is True and not Keyword.get('Http_Keyfile' , '') == '' else Const.Http_Keyfile ,
                Certfile = Keyword.get('Http_Certfile') if isinstance(Keyword.get('Http_Certfile') , str) is True and not Keyword.get('Http_Certfile' , '') == '' else Const.Http_Certfile ,
                Callable = Keyword.get('Http_Callable') if callable(Keyword.get('Http_Callable')) is True else Const.Http_Callable ,
            ) ,
            Websocket = Library.Simple(
                Status = Keyword.get('Websocket_Status') if isinstance(Keyword.get('Websocket_Status') , bool) is True else Const.Websocket_Status ,
                Print = Keyword.get('Websocket_Print') if isinstance(Keyword.get('Websocket_Print') , bool) is True else Const.Websocket_Print ,
                Journal = Keyword.get('Websocket_Journal') if isinstance(Keyword.get('Websocket_Journal') , bool) is True else Const.Websocket_Journal ,
                Cpu = int(Keyword.get('Websocket_Cpu') if isinstance(Keyword.get('Websocket_Cpu') , int) is True and Keyword.get('Websocket_Cpu') > 0 else Const.Websocket_Cpu) ,
                Port = Keyword.get('Websocket_Port') if isinstance(Keyword.get('Websocket_Port') , list) is True and len(Keyword.get('Websocket_Port')) > 0 else Const.Websocket_Port ,
                Timeout = int(Keyword.get('Websocket_Timeout') if isinstance(Keyword.get('Websocket_Timeout') , int) is True and Keyword.get('Websocket_Timeout') > 0 else Const.Websocket_Timeout) ,
                Connect = int(Keyword.get('Websocket_Connect') if isinstance(Keyword.get('Websocket_Connect') , int) is True and Keyword.get('Websocket_Connect') > 0 else Const.Websocket_Connect) ,
                Keyfile = Keyword.get('Websocket_Keyfile') if isinstance(Keyword.get('Websocket_Keyfile') , str) is True and not Keyword.get('Websocket_Keyfile' , '') == '' else Const.Websocket_Keyfile ,
                Certfile = Keyword.get('Websocket_Certfile') if isinstance(Keyword.get('Websocket_Certfile') , str) is True and not Keyword.get('Websocket_Certfile' , '') == '' else Const.Websocket_Certfile ,
                Callable = Keyword.get('Websocket_Callable') if callable(Keyword.get('Websocket_Callable')) is True else Const.Websocket_Callable ,
            ) ,
            Resource = Library.Simple(
                Status = Keyword.get('Resource_Status') if isinstance(Keyword.get('Resource_Status') , bool) is True else Const.Resource_Status ,
                Print = Keyword.get('Resource_Print') if isinstance(Keyword.get('Resource_Print') , bool) is True else Const.Resource_Print ,
                Journal = Keyword.get('Resource_Journal') if isinstance(Keyword.get('Resource_Journal') , bool) is True else Const.Resource_Journal ,
                Sleep = int(Keyword.get('Resource_Sleep') if isinstance(Keyword.get('Resource_Sleep') , int) is True and Keyword.get('Resource_Sleep') > 0 else Const.Resource_Sleep) ,
                Cpu = Keyword.get('Resource_Cpu') if isinstance(Keyword.get('Resource_Cpu') , bool) is True else Const.Resource_Cpu ,
                Memory = Keyword.get('Resource_Memory') if isinstance(Keyword.get('Resource_Memory') , bool) is True else Const.Resource_Memory ,
                Network = Keyword.get('Resource_Network') if isinstance(Keyword.get('Resource_Network') , bool) is True else Const.Resource_Network ,
                Disk = Keyword.get('Resource_Disk') if isinstance(Keyword.get('Resource_Disk') , bool) is True else Const.Resource_Disk ,
                File = Keyword.get('Resource_File') if isinstance(Keyword.get('Resource_File') , bool) is True else Const.Resource_File ,
                Load = Keyword.get('Resource_Load') if isinstance(Keyword.get('Resource_Load') , bool) is True else Const.Resource_Load ,
            ) ,
            Journal = Library.Simple(
                Thread = int(Keyword.get('Journal_Thread') if isinstance(Keyword.get('Journal_Thread') , int) is True and Keyword.get('Journal_Thread') > 0 else Const.Journal_Thread) ,
                Clean = int(Keyword.get('Journal_Clean') if isinstance(Keyword.get('Journal_Clean') , int) is True and Keyword.get('Journal_Clean') > 0 else Const.Journal_Clean) ,
            ) ,
            Runtime = Library.Simple(
                Trace = Keyword.get('Runtime_Trace') if isinstance(Keyword.get('Runtime_Trace') , bool) is True else Const.Runtime_Trace ,
                Http = Keyword.get('Runtime_Http') if isinstance(Keyword.get('Runtime_Http') , bool) is True else Const.Runtime_Http ,
                Websocket = Keyword.get('Runtime_Websocket') if isinstance(Keyword.get('Runtime_Websocket') , bool) is True else Const.Runtime_Websocket ,
                Resource = Keyword.get('Runtime_Resource') if isinstance(Keyword.get('Runtime_Resource') , bool) is True else Const.Runtime_Resource ,
            ) ,
        );


from . import Library;
from .Function import Function;

class Journal :
    
    @classmethod
    def Lunch(Self , Context:Library.Simple) -> None :
        Function.Proctitle(f'{Context.Config.Proctitle}.Journal');
        if Self.Check(Context) is False : Function.Print(f'{Context.Config.Proctitle}.Journal Unable To Listen, [Http | Websocket | Resource] One Of Them Must Be Activated');
        else :
            Process:list[Library.Processing.Process] = list();
            Process.append(Library.Processing.Process(target = Self.Log , args = (Context , )));
            Process.append(Library.Processing.Process(target = Self.Clean , args = (Context , )));
            for X in Process : X.start();
            for X in Process : X.join();
    
    @staticmethod
    def Check(Context:Library.Simple) -> bool :
        return Context.Config.Http.Journal | Context.Config.Websocket.Journal | Context.Config.Resource.Journal;
    
    @classmethod
    def Log(Self , Context) :
        Function.Proctitle(f'{Context.Config.Proctitle}.Journal.Log');
        Action = list();
        if Context.Config.Http.Journal is True : Action.append('Http');
        if Context.Config.Websocket.Journal is True : Action.append('Websocket');
        if Context.Config.Resource.Journal is True : Action.append('Resource');
        with Library.Executor(max_workers = Context.Config.Journal.Thread) as Executor :
            Function.Print(f'{Context.Config.Proctitle}.Journal Log Start Up, Listen In | Pid[{Library.Os.getpid()}] | [{" | ".join(Action)}]');
            while True :
                if Context.Journal.qsize() == 0 : continue;
                Source = Context.Journal.get();
                Executor.submit(getattr(Self , Source.Method) , Source , Context);
    
    @classmethod
    def Trace(Self , Source:Library.Simple , Context:Library.Simple) -> None :
        Self.Save(Source.Body , Library.Simple(
            Print = True ,
            Path = Context.Config.Runtime.Trace ,
        ));
    
    @classmethod
    def Http(Self , Source:Library.Simple , Context:Library.Simple) -> None :
        Log = list();
        if Source.Type == 'Request' : Log = Self.Request(Source.Body);
        elif Source.Type == 'Response' : Log = Self.Response(Source.Body);
        if not len(Log) == 0 : Self.Save('\n'.join(Log) , Library.Simple(
            Print = Context.Config.Http.Print ,
            Path = Context.Config.Runtime.Http ,
        ));
    
    @classmethod
    def Websocket(Self , Source:Library.Simple , Context:Library.Simple) -> None :
        Log = list();
        if Source.Type == 'Request' : Log = Self.Request(Source.Body);
        elif Source.Type == 'Response' : Log = Self.Response(Source.Body);
        if not len(Log) == 0 : Self.Save('\n'.join(Log) , Library.Simple(
            Print = Context.Config.Websocket.Print ,
            Path = Context.Config.Runtime.Websocket ,
        ));
    
    @classmethod
    def Resource(Self , Source:Library.Simple , Context:Library.Simple) -> None :
        Self.Save('\n'.join(Source.Body) , Library.Simple(
            Print = Context.Config.Resource.Print ,
            Path = Context.Config.Runtime.Resource ,
        ));
    
    @classmethod
    def Request(Self , Source:Library.Simple) -> list :
        Log = list();
        try :
            Log.append(' | '.join([
                'Uuid' ,
                f'[{Source.Uuid}]' ,
            ]));
            Log.append(' | '.join([
                'Socket' ,
                f'[{Source.Socket}]' ,
            ]));
            Log.append(' | '.join([
                'Pid' ,
                f'Server[{Source.Server}]' ,
                f'Worker[{Source.Worker}]' ,
            ]));
            Log.append('----' * 10);
            Log.append(' | '.join([
                'Network' ,
            ]));
            Log.append(' | '.join([
                'Location' ,
                f'Ip[{Source.Location[0]}]' ,
                f'Port[{Source.Location[1]}]' ,
            ]));
            Log.append(' | '.join([
                'Remote' ,
                f'Ip[{Source.Remote[0]}]' ,
                f'Port[{Source.Remote[1]}]' ,
            ]));
            Log.append('----' * 10);
            Log.append(' | '.join([
                'Cors' ,
                f'[{Source.Request.Cors}]' ,
            ]));
            Log.append(' | '.join([
                'Method' ,
                f'[{Source.Request.Method}]' ,
            ]));
            Log.append(' | '.join([
                'Url' ,
                f'[{Source.Request.Url}]' ,
            ]));
            Log.append(' | '.join([
                'Path' ,
                f'[{Source.Request.Path}]' ,
            ]));
            Log.append('----' * 10);
            Log.append('Query');
            for X in Source.Request.Query : Log.append(' | '.join([
                X ,
                f'[{Library.Json.dumps(Source.Request.Query.get(X)).decode()}]' ,
            ]));
            Log.append('----' * 10);
            Log.append('Body');
            if isinstance(Source.Request.Body , str) : Log.append(Source.Request.Body);
            else :
                for X in Source.Request.Body : Log.append(' | '.join([
                    X ,
                    f'[{Library.Json.dumps(Source.Request.Body.get(X)).decode()}]' ,
                ]));
            Log.append('----' * 10);
            Log.append('Header');
            for X in Source.Request.Header : Log.append(' | '.join([
                X ,
                f'[{Library.Json.dumps(Source.Request.Header.get(X)).decode()}]' ,
            ]));
            Log.append('----' * 10);
            Log.append('Original');
            for X in Source.Request.Original.decode().split('\r\n') :
                if X : Log.append(X);
        except : Function.Trace();
        finally : return Self.Change(Log);
    
    @classmethod
    def Response(Self , Source:Library.Simple) -> list :
        Log = list();
        try :
            Log.append(' | '.join([
                'Uuid' ,
                f'[{Source.Uuid}]' ,
            ]));
            for X in Source.Response : Log.append(' | '.join([
                X ,
                f'[{Source.Response.get(X)}]' ,
            ]));
        except : Function.Trace();
        finally : return Self.Change(Log);
    
    @staticmethod
    def Change(Source:list) -> list :
        Log = list();
        if not len(Source) == 0 :
            for Item in Source :
                Log.append(f'|-- {Item}');
        return Log;
    
    @staticmethod
    def Save(Source:str , Config:Library.Simple) -> None :
        try :
            Calendar = Function.Time(Method = '%Y-%m-%d');
            Hour = Function.Time(Method = '%H');
            Minute = Function.Time(Method = '%M');
            Path = Library.Os.path.realpath('/'.join([
                Library.Os.getcwd() ,
                Config.Path ,
                Calendar ,
                Hour ,
            ]));
            if Library.Os.path.exists(Path) is False : Library.Os.makedirs(Path , exist_ok = True);
            Log = '\n'.join([
                f'|## {"##" * 50}' ,
                f'|| {Function.Time(Method = True)}' ,
                '' ,
                Source ,
                f'|## {"##" * 50}' ,
                '' ,
            ]);
            with open(Library.Os.path.normpath(f'{Path}/{Minute}.log') , 'a' , encoding = 'utf-8') as File : Function.Print(Log , File = File);
            if Config.Print is True : Function.Print(Log);
        except : Function.Trace();
    
    @classmethod
    def Clean(Self , Context) :
        Function.Proctitle(f'{Context.Config.Proctitle}.Journal.Clean');
        Function.Print(f'{Context.Config.Proctitle}.Journal Clean Start Up, Listen In | Pid[{Library.Os.getpid()}] | [{Context.Config.Journal.Clean}] Days');
        Function.Policy();
        while True :
            if int(Function.Time(Method = '%H')) == 1 : Library.Asyncio.gather([
                Self.Remote(Context.Config.Runtime.Http , Context.Config.Journal.Clean) ,
                Self.Remote(Context.Config.Runtime.Websocket , Context.Config.Journal.Clean) ,
                Self.Remote(Context.Config.Runtime.Resource , Context.Config.Journal.Clean) ,
                Self.Remote(Context.Config.Runtime.Trace , Context.Config.Journal.Clean) ,
            ]);
            Library.Time.sleep(Self.Sleep());
    
    @staticmethod
    def Sleep() :
        Current = Function.Time();
        Tomorrow = (Current + Library.Delta(days = 1)).replace(hour = 1 , minute = 0 , second = 0 , microsecond = 0);
        return int((Tomorrow - Current).total_seconds());
    
    @staticmethod
    async def Remote(Path , Day) :
        Route = Library.Os.path.realpath('/'.join([
            Library.Os.getcwd() ,
            Path ,
        ]));
        if Library.Os.path.exists(Route) is False : return False;
        Delta = Function.Time() - Library.Delta(days = Day);
        for Dir in Library.Os.listdir(Route) :
            Base = Library.Os.path.join(Route , Dir);
            if Library.Os.path.isdir(Base) is False : continue;
            try :
                Date = Library.Date.strptime(Dir , '%Y-%m-%d');
                if Date < Delta : Library.Shutil.rmtree(Base);
            except : continue;
        await Library.Asyncio.sleep(0);


from . import Library , Const;
from .Function import Function;

class Control :
    
    @classmethod
    def Lunch(Self , Context:Library.Simple) -> None :
        Function.Proctitle(Context.Signal.Proctitle);
        Process = [Library.Processing.Process(target = Self.Server , args = (Context , Port)) for Port in Context.Signal.Config.Port];
        for X in Process : X.start();
        for X in Process : X.join();
    
    @classmethod
    def Server(Self , Context:Library.Simple , Port:int) -> None :
        Function.Proctitle(f'{Context.Signal.Proctitle}.Server');
        Socket = Library.Socket.socket(Library.Socket.AF_INET , Library.Socket.SOCK_STREAM);
        Socket.setsockopt(Library.Socket.SOL_SOCKET , Library.Socket.SO_REUSEADDR , 1);
        try : Socket.setsockopt(Library.Socket.SOL_SOCKET , Library.Socket.SO_REUSEPORT , 1);
        except : pass;
        Socket.setsockopt(Library.Socket.IPPROTO_TCP , Library.Socket.TCP_NODELAY , 1);
        Socket.setsockopt(Library.Socket.SOL_SOCKET , Library.Socket.SO_SNDBUF , 1 << 20);
        Socket.setsockopt(Library.Socket.SOL_SOCKET , Library.Socket.SO_RCVBUF , 1 << 20);
        Socket.bind((Const.Socket_Host , Port));
        Socket.listen(Const.Socket_Listen);
        Socket.setblocking(False);
        if Library.Platform.system().lower() == 'linux' :
            Process = [Library.Processing.Process(target = Self.Worker , args = (Context , Socket) , daemon = True) for _ in range(Context.Signal.Config.Cpu)];
            for X in Process : X.start();
            for X in Process : X.join();
        else : Self.Worker(Context , Socket);
    
    @classmethod
    def Worker(Self , Context:Library.Simple , Socket:Library.Socket.socket) -> None :
        Function.Proctitle(f'{Context.Signal.Proctitle}.Worker');
        Function.Policy();
        Function.Print(f'{Context.Signal.Proctitle} Start Up, Listen In | Pid[{Library.Os.getpid()}] | Port[{Socket.getsockname()[1]}]');
        Library.Asyncio.run(Self.Running(Context , Socket));
    
    @classmethod
    async def Running(Self , Context:Library.Simple , Socket:Library.Socket.socket) -> None :
        Service = await Library.Asyncio.start_server(
            Library.Functools.partial(Self.Handle , Context) ,
            sock = Socket ,
            reuse_port = True ,
            backlog = Const.Socket_Listen ,
            ssl = Function.Ssl(Context.Signal.Config) ,
        );
        async with Service : await Service.serve_forever();
    
    @staticmethod
    async def Handle(Context:Library.Simple , Reader:Library.Asyncio.StreamReader , Writer:Library.Asyncio.StreamWriter) -> None :
        try :
            Socket = Writer.get_extra_info('socket');
            Context.Signal.Uuid = Function.Uuid();
            Context.Signal.Socket = Socket.fileno();
            Context.Signal.Server = Library.Os.getppid();
            Context.Signal.Worker = Library.Os.getpid();
            Context.Signal.Location = list(Socket.getsockname());
            Context.Signal.Remote = list(Socket.getpeername());
            await Context.Signal.Handle(Context , Reader , Writer);
        except : Function.Trace(Context.Journal);
        finally :
            try :
                if Writer.is_closing() is False :
                    Writer.close();
                    await Writer.wait_closed();
            except : pass;
            finally : await Library.Asyncio.sleep(0);
    
    @staticmethod
    async def Request(Context , Reader:Library.Asyncio.StreamReader) -> Library.Simple :
        try :
            Context.Signal.Request = Library.Simple();
            Context.Signal.Request.Status = False;
            Context.Signal.Request.Cors = False;
            Context.Signal.Request.Original = Const.Socket_Empty;
            Context.Signal.Request.Query = dict();
            Context.Signal.Request.Header = dict();
            Context.Signal.Request.Body = dict();
            Context.Signal.Response = dict();
            try : Head = await Library.Asyncio.wait_for(Reader.readline() , timeout = Const.Socket_Waittime);
            except : raise;
            Context.Signal.Request.Original += Head;
            Decode = Head.decode(errors = 'ignore').strip();
            try : Context.Signal.Request.Method , Context.Signal.Request.Url , _ = Decode.split();
            except : raise;
            while True :
                try :
                    try : Header = await Library.Asyncio.wait_for(Reader.readline() , timeout = Const.Socket_Waittime);
                    except : raise;
                    if Header in (Const.Socket_Empty , Const.Socket_Newline) : raise;
                    Context.Signal.Request.Original += Header;
                    Decode = Header.decode(errors = 'ignore').strip();
                    if ':' not in Decode : continue;
                    Primary , Data = Decode.split(':' , 1);
                    if Primary.lower() == 'cookie' :
                        Cookie = Library.Cookie();
                        Cookie.load(Data);
                        Context.Signal.Request.Header['cookie'] = {Key.lower().strip() : Value.value.strip() for Key , Value in Cookie.items()};
                    else : Context.Signal.Request.Header[Primary.lower().strip()] = Data.strip();
                except : break;
            if any([
                Context.Signal.Request.Header.get('sec-fetch-mode' , 'none').lower() == 'cors' ,
                'referer' in Context.Signal.Request.Header ,
            ]) is True : Context.Signal.Request.Cors = True;
            Parse = Library.Parse.urlparse(Context.Signal.Request.Url);
            Context.Signal.Request.Path = Parse.path;
            Query = Library.Parse.parse_qs(Parse.query , keep_blank_values = True);
            if not len(Query) == 0 : Context.Signal.Request.Query = {Key.strip() : Value[0].strip() if len(Value) == 1 else Value for Key , Value in Query.items()};
            Context.Signal.Request.Length = int(Context.Signal.Request.Header.get('content-length' , 0));
            if all([
                not Context.Signal.Request.Length == 0 ,
                Context.Signal.Request.Method == 'POST' ,
            ]) is True :
                try : Body = await Library.Asyncio.wait_for(Reader.readexactly(Context.Signal.Request.Length) , timeout = Const.Socket_Waittime);
                except : raise;
                Context.Signal.Request.Original += Const.Socket_Newline + Body;
                try : Context.Signal.Request.Body = Library.Json.loads(Body);
                except :
                    Data = Body.decode();
                    Parse = Library.Parse.parse_qs(Data);
                    if len(Parse) == 0 : Context.Signal.Request.Body = Data;
                    else : Context.Signal.Request.Body = {Key.strip() : Value[0].strip() if len(Value) == 1 else Value for Key , Value in Parse.items()};
            Context.Signal.Request.Status = True;
        except : pass;
        finally : return Context;
    
    @staticmethod
    def Business(Context:Library.Simple) -> dict :
        Source = dict(
            Code = 200 ,
            Body = str() ,
        );
        if Context.Signal.Config.Callable is None : Source['Code'] = 404;
        else :
            Response = Context.Signal.Config.Callable(Library.Simple(
                Network = Context.Signal.Remote ,
                Request = Context.Signal.Request ,
            ));
            if all([
                isinstance(Response , dict) is True ,
                'Body' in Response ,
            ]) is True : Source.update(Response);
            else : Source['Body'] = Response;
        return Source;


from . import Library , Const;
from .Function import Function;
from .Control import Control;

class Websocket :
    
    @classmethod
    def Lunch(Self , Context:Library.Simple) -> None :
        Context.Signal = Library.Simple(
            Proctitle = f'{Context.Config.Proctitle}.Websocket' ,
            Config = Context.Config.Websocket ,
            Handle = Self.Client ,
        );
        Control.Lunch(Context);
    
    @classmethod
    async def Client(Self , Context:Library.Simple , Reader:Library.Asyncio.StreamReader , Writer:Library.Asyncio.StreamWriter) -> None :
        try :
            while True :
                Context = await Control.Request(Context , Reader);
                if Context.Signal.Request.Status is False : break;
                if len(Context.Signal.Request.Header) == 0 : raise ValueError(402);
                if Context.Signal.Request.Header.get('upgrade' , str()).lower() == 'websocket' :
                    await Self.Upgrade(Context , Writer);
                    await Self.Session(Context , Reader , Writer);
        except ValueError as Error :
            try :
                Context.Signal.Response = dict(Code = Error.args[0]);
                await Self.Send(Context , Writer);
            except : Function.Trace(Context.Journal);
        except : Function.Trace(Context.Journal);
    
    @staticmethod
    async def Upgrade(Context:Library.Simple , Writer:Library.Asyncio.StreamWriter) -> None :
        try :
            if Writer.is_closing() is False :
                Writer.write(Const.Socket_Upgrade % (
                    Function.Hash(f'{Context.Signal.Request.Header.get("sec-websocket-key")}{Const.Socket_Guid}').encode() ,
                ));
                await Writer.drain();
        except : Function.Trace(Context.Journal);
        finally : await Library.Asyncio.sleep(0);
    
    @classmethod
    async def Session(Self , Context:Library.Simple , Reader:Library.Asyncio.StreamReader , Writer:Library.Asyncio.StreamWriter) -> None :
        try :
            Context.Begin = Function.Time(Int = False);
            Queue = Library.Asyncio.Queue();
            Waiting = Library.Asyncio.create_task(Self.Worker(Queue , Context.Journal , Writer));
            while True :
                if (Function.Time(Int = False) - Context.Begin) > Context.Signal.Config.Connect : break;
                if Context.Signal.Config.Timeout > 0 :
                    try : Header = await Library.Asyncio.wait_for(Reader.readexactly(2) , timeout = Context.Signal.Config.Timeout);
                    except : break;
                else : Header = await Reader.readexactly(2);
                Fin = Header[0] & Const.Socket_Fin;
                Opcode = Header[0] & Const.Socket_Opcode;
                Masked = Header[1] & Const.Socket_Fin;
                Payload = Header[1] & Const.Socket_Payload;
                if Payload == 126 : Payload = Library.Struct.unpack(Const.Socket_H , await Reader.readexactly(2))[0];
                if Payload == 127 : Payload = Library.Struct.unpack(Const.Socket_Q , await Reader.readexactly(8))[0];
                Mask = await Reader.readexactly(4) if Masked else Const.Socket_Empty;
                Load = await Reader.readexactly(Payload);
                if Masked : Load = bytes(Byte ^ Mask[Index % 4] for Index , Byte in enumerate(Load));
                if Opcode in (1 , 2) :
                    try : Context.Signal.Request.Body = Library.Json.loads(Load);
                    except :
                        Body = Load.decode();
                        Parse = Library.Parse.parse_qs(Body);
                        if len(Parse) == 0 : Context.Signal.Request.Body = Body;
                        else : Context.Signal.Request.Body = {Key.strip() : Value[0].strip() if len(Value) == 1 else Value for Key , Value in Parse.items()};
                    finally :
                        if Context.Signal.Config.Journal is True : Context.Journal.put(Library.Simple(
                            Method = 'Websocket' ,
                            Type = 'Request' ,
                            Body = Context.Signal ,
                        ));
                        await Queue.put(Library.Copy.deepcopy(Context.Signal));
                else : break;
            await Queue.put(None);
            await Waiting;
        except ValueError as Error :
            try :
                Context.Signal.Response = dict(Code = Error.args[0]);
                await Self.Send(Context , Writer);
            except : Function.Trace(Context.Journal);
        except : Function.Trace(Context.Journal);
        finally : await Library.Asyncio.sleep(0);
    
    @classmethod
    async def Worker(Self , Queue:Library.Asyncio.Queue , Journal:Library.Processing.Queue , Writer:Library.Asyncio.StreamWriter) -> None :
        while True :
            Context = await Queue.get();
            if Context is None : break;
            await Self.Task(Library.Simple(
                Signal = Context ,
                Journal = Journal ,
            ) , Writer);
    
    @classmethod
    async def Task(Self , Context:Library.Simple , Writer:Library.Asyncio.StreamWriter) -> None :
        try :
            Context.Signal.Response = await Library.Asyncio.to_thread(Control.Business , Context);
            await Self.Send(Context , Writer);
        except :
            try :
                Context.Signal.Response = dict(Code = 503);
                await Self.Send(Context , Writer);
            except : pass;
            finally : Function.Trace(Context.Journal);
    
    @staticmethod
    async def Send(Context:Library.Simple , Writer:Library.Asyncio.StreamWriter) -> None :
        try :
            if Writer.is_closing() is False :
                Code = Context.Signal.Response.get('Code' , 200);
                Body = Context.Signal.Response.get('Body' , str());
                Response_Code = getattr(Const , f'Socket_{Code}' , Const.Socket_400);
                if all([
                    not Code == 200 ,
                    any([
                        Body == '' ,
                        Body is None ,
                        len(Body) == 0 ,
                    ]) ,
                ]) is True : Body = Response_Code;
                if Body is None : Body = Const.Socket_Empty.decode();
                if isinstance(Body , bytes) is True : Body = Body.decode();
                Data = Library.Json.dumps(dict(
                    Code = Code ,
                    Body = Body ,
                ));
                Header = bytearray();
                Header.append(0x81);
                Length = len(Data);
                if Length <= 125 : Header.append(Length);
                elif Length < 65535 :
                    Header.append(126);
                    Header += Library.Struct.pack(Const.Socket_H , Length);
                else :
                    Header.append(127);
                    Header += Library.Struct.pack(Const.Socket_Q , Length);
                Writer.write(Header + Data);
                await Writer.drain();
                if Context.Signal.Config.Journal is True : Context.Journal.put(Library.Simple(
                    Method = 'Websocket' ,
                    Type = 'Response' ,
                    Body = Context.Signal ,
                ));
        except : Function.Trace(Context.Journal);
        finally : await Library.Asyncio.sleep(0);
    


from . import Library;
from .Function import Function;

CPU = False;
MEMORY = False;
NETWORK = False;
DISK = False;
FILE = False;
LOAD = False;
JOURNAL = False;
DURATION_PERFORMANCE_CPU = Library.Deque(maxlen = 2);
DURATION_PERFORMANCE_MEMORY = Library.Deque(maxlen = 2);
DURATION_PERFORMANCE_NETWORK = Library.Deque(maxlen = 2);
DURATION_PERFORMANCE_DISK = Library.Deque(maxlen = 2);
DURATION_PERFORMANCE_FILE = Library.Deque(maxlen = 2);
DURATION_PERFORMANCE_LOAD = Library.Deque(maxlen = 2);
DURATION_STATISTICS_CPU = Library.Deque(maxlen = 2);
DURATION_STATISTICS_MEMORY = Library.Deque(maxlen = 2);
DURATION_STATISTICS_NETWORK = Library.Deque(maxlen = 2);
DURATION_STATISTICS_DISK = Library.Deque(maxlen = 2);
DURATION_STATISTICS_FILE = Library.Deque(maxlen = 2);
DURATION_STATISTICS_LOAD = Library.Deque(maxlen = 2);

class Resource :
    
    @classmethod
    def Lunch(Self , Context:Library.Simple) -> None :
        Function.Proctitle(f'{Context.Config.Proctitle}.Resource');
        if Self.Check(Context) is True :
            Action = list();
            if CPU is True : Action.append('Cpu');
            if MEMORY is True : Action.append('Memory');
            if NETWORK is True : Action.append('Network');
            if DISK is True : Action.append('Disk');
            if FILE is True : Action.append('File');
            if LOAD is True : Action.append('Load');
            Function.Print(f'{Context.Config.Proctitle}.Resource Start Up, Listen In | Pid[{Library.Os.getpid()}] | [{" | ".join(Action)}]');
            Self.Performance(Context);
        else : Function.Print(f'{Context.Config.Proctitle}.Resource Unable To Listen, [Cpu | Memory | Network | Disk | File | Load] One Of Them Must Be Activated');
    
    @staticmethod
    def Check(Context:Library.Simple) -> bool :
        global CPU , MEMORY , NETWORK , DISK , FILE , LOAD , JOURNAL;
        if Context.Config.Resource.Cpu is True : CPU = True;
        if Context.Config.Resource.Memory is True : MEMORY = True;
        if Context.Config.Resource.Network is True : NETWORK = True;
        if Context.Config.Resource.Disk is True : DISK = True;
        if Context.Config.Resource.File is True : FILE = True;
        if Context.Config.Resource.Load is True and Library.Platform.system().lower() == 'linux' : LOAD = True;
        if Context.Config.Resource.Journal is True : JOURNAL = True;
        return CPU | MEMORY | NETWORK | DISK | FILE | LOAD;
    
    @staticmethod
    def Performance(Context:Library.Simple) -> None :
        while True :
            Library.Time.sleep(Context.Config.Resource.Sleep);
            try :
                if CPU is True : DURATION_PERFORMANCE_CPU.append(Performance.Cpu());
                if MEMORY is True : DURATION_PERFORMANCE_MEMORY.append(Performance.Memory());
                if NETWORK is True : DURATION_PERFORMANCE_NETWORK.append(Performance.Network());
                if DISK is True : DURATION_PERFORMANCE_DISK.append(Performance.Disk());
                if FILE is True : DURATION_PERFORMANCE_FILE.append(Performance.File());
                if LOAD is True : DURATION_PERFORMANCE_LOAD.append(Performance.Load());
                if len(DURATION_PERFORMANCE_CPU) == 2 : DURATION_STATISTICS_CPU.append(Statistics.Cpu());
                if len(DURATION_PERFORMANCE_MEMORY) == 2 : DURATION_STATISTICS_MEMORY.append(Statistics.Memory());
                if len(DURATION_PERFORMANCE_NETWORK) == 2 : DURATION_STATISTICS_NETWORK.append(Statistics.Network());
                if len(DURATION_PERFORMANCE_DISK) == 2 : DURATION_STATISTICS_DISK.append(Statistics.Disk());
                if len(DURATION_PERFORMANCE_FILE) == 2 : DURATION_STATISTICS_FILE.append(Statistics.File());
                if len(DURATION_PERFORMANCE_LOAD) == 2 : DURATION_STATISTICS_LOAD.append(Statistics.Load());
                if JOURNAL is True : Journal.Create(Context);
            except : Function.Trace(Context.Journal);

class Journal :
    
    @classmethod
    def Create(Self , Context:Library.Simple) -> None :
        Source = list();
        if len(DURATION_STATISTICS_CPU) == 2 : Source.append(Self.Cpu());
        if len(DURATION_STATISTICS_MEMORY) == 2 : Source.append(Self.Memory());
        if len(DURATION_STATISTICS_NETWORK) == 2 : Source.append(Self.Network());
        if len(DURATION_STATISTICS_DISK) == 2 : Source.append(Self.Disk());
        if len(DURATION_STATISTICS_FILE) == 2 : Source.append(Self.File());
        if len(DURATION_STATISTICS_LOAD) == 2 : Source.append(Self.Load());
        if not len(Source) == 0 :
            Log = list();
            for Item in Source :
                Log.append(f'|== {"==" * 20}');
                for Index , Value in enumerate(Item) : Log.append(f'|{"--" if Index == 0 else "----"} {Value}');
            Log.append(f'|== {"==" * 20}');
            Context.Journal.put(Library.Simple(
                Method = 'Resource' ,
                Body = Log ,
            ));
    
    @staticmethod
    def Cpu() -> list :
        Statistics_Previous , Statistics_Currently = DURATION_STATISTICS_CPU;
        Performance_Previous , Performance_Currently = DURATION_PERFORMANCE_CPU;
        Source = list();
        Source.append('中央处理器');
        Source.append('----' * 10);
        Source.append(' | '.join([
            '系统进程' ,
            f'增长[{Statistics_Currently.Process}]个' ,
            f'趋势[{Statistics_Currently.Process - Statistics_Previous.Process}]个' ,
            f'本次[{Performance_Currently.Process}]个' ,
            f'上次[{Performance_Previous.Process}]个' ,
        ]));
        Source.append('----' * 10);
        Source.append(' | '.join([
            '系统线程' ,
            f'增长[{Statistics_Currently.Thread}]个' ,
            f'趋势[{Statistics_Currently.Thread - Statistics_Previous.Thread}]个' ,
            f'本次[{Performance_Currently.Thread}]个' ,
            f'上次[{Performance_Previous.Thread}]个' ,
        ]));
        Source.append('----' * 10);
        Source.append(' | '.join([
            '核心使用率' ,
            f'增长[{Statistics_Currently.Core}]%' ,
            f'趋势[{Statistics_Currently.Core - Statistics_Previous.Core}]%' ,
            f'本次[{Performance_Currently.Core}]%' ,
            f'上次[{Performance_Previous.Core}]%' ,
        ]));
        Source.append('----' * 10);
        for Index , Value in enumerate(Statistics_Currently.Respective) : Source.append(' | '.join([
            f'内核[{Index + 1}]' ,
            f'增长[{Value}]%' ,
            f'趋势[{Value - Statistics_Previous.Respective[Index]}]%' ,
            f'本次[{Performance_Currently.Respective[Index]}]%' ,
            f'上次[{Performance_Previous.Respective[Index]}]%' ,
        ]));
        for Pid in Statistics_Currently.Expense :
            if Pid in Performance_Currently.Expense and Pid in Performance_Previous.Expense and Pid in Statistics_Previous.Expense :
                Source.append('----' * 10);
                Source.append(' | '.join([
                    f'程序[{Pid}]' ,
                    f'名称[{Statistics_Currently.Expense[Pid].Name}]' ,
                ]));
                Source.append(' | '.join([
                    f'CPU' ,
                    f'增长[{Statistics_Currently.Expense[Pid].Cpu}]%' ,
                    f'趋势[{Statistics_Currently.Expense[Pid].Cpu - Statistics_Previous.Expense[Pid].Cpu}]%' ,
                    f'本次[{Performance_Currently.Expense[Pid].Cpu}]%' ,
                    f'上次[{Performance_Previous.Expense[Pid].Cpu}]%' ,
                ]));
                Source.append(' | '.join([
                    f'MEMORY' ,
                    f'增长[{Statistics_Currently.Expense[Pid].Memory}]%' ,
                    f'趋势[{Statistics_Currently.Expense[Pid].Memory - Statistics_Previous.Expense[Pid].Memory}]%' ,
                    f'本次[{Performance_Currently.Expense[Pid].Memory}]%' ,
                    f'上次[{Performance_Previous.Expense[Pid].Memory}]%' ,
                ]));
        return Source;
    
    @staticmethod
    def Memory() -> list :
        Statistics_Previous , Statistics_Currently = DURATION_STATISTICS_MEMORY;
        Performance_Previous , Performance_Currently = DURATION_PERFORMANCE_MEMORY;
        Source = list();
        Source.append('系统内存');
        Source.append(' | '.join([
            '总内存' ,
            f'[{Performance_Currently.Total}]Mb' ,
        ]));
        Source.append(' | '.join([
            '使用率' ,
            f'增长[{Statistics_Currently.Percent}]%' ,
            f'趋势[{Statistics_Currently.Percent - Statistics_Previous.Percent}]%' ,
            f'本次[{Performance_Currently.Percent}]%' ,
            f'上次[{Performance_Previous.Percent}]%' ,
        ]));
        Source.append(' | '.join([
            '已使用' ,
            f'增长[{Statistics_Currently.Used}]%' ,
            f'趋势[{Statistics_Currently.Used - Statistics_Previous.Used}]%' ,
            f'本次[{Performance_Currently.Used}]Mb' ,
            f'上次[{Performance_Previous.Used}]Mb' ,
        ]));
        Source.append(' | '.join([
            '可用' ,
            f'增长[{Statistics_Currently.Available}]%' ,
            f'趋势[{Statistics_Currently.Available - Statistics_Previous.Available}]%' ,
            f'本次[{Performance_Currently.Available}]Mb' ,
            f'上次[{Performance_Previous.Available}]Mb' ,
        ]));
        Source.append(' | '.join([
            '缓存' ,
            f'增长[{Statistics_Currently.Cached}]%' ,
            f'趋势[{Statistics_Currently.Cached - Statistics_Previous.Cached}]%' ,
            f'本次[{Performance_Currently.Cached}]Mb' ,
            f'上次[{Performance_Previous.Cached}]Mb' ,
        ]));
        return Source;
    
    @staticmethod
    def Network() -> list :
        Statistics_Previous , Statistics_Currently = DURATION_STATISTICS_NETWORK;
        Performance_Previous , Performance_Currently = DURATION_PERFORMANCE_NETWORK;
        Source = list();
        Source.append('系统网络');
        for X in Statistics_Currently.Send.Bytes :
            Source.append('----' * 10);
            Source.append(f'网卡[{X}]');
            Source.append(' | '.join([
                '发送字节' ,
                f'增长[{Statistics_Currently.Send.Bytes[X]}]%' ,
                f'趋势[{Statistics_Currently.Send.Bytes[X] - Statistics_Previous.Send.Bytes[X]}]%' ,
                f'本次[{Performance_Currently.Send.Bytes[X]}]bytes' ,
                f'上次[{Performance_Previous.Send.Bytes[X]}]bytes' ,
            ]));
            Source.append(' | '.join([
                '发送包数' ,
                f'增长[{Statistics_Currently.Send.Packets[X]}]%' ,
                f'趋势[{Statistics_Currently.Send.Packets[X] - Statistics_Previous.Send.Packets[X]}]%' ,
                f'本次[{Performance_Currently.Send.Packets[X]}]个' ,
                f'上次[{Performance_Previous.Send.Packets[X]}]个' ,
            ]));
            Source.append(' | '.join([
                '接收字节' ,
                f'增长[{Statistics_Currently.Receive.Bytes[X]}]%' ,
                f'趋势[{Statistics_Currently.Receive.Bytes[X] - Statistics_Previous.Receive.Bytes[X]}]%' ,
                f'本次[{Performance_Currently.Receive.Bytes[X]}]bytes' ,
                f'上次[{Performance_Previous.Receive.Bytes[X]}]bytes' ,
            ]));
            Source.append(' | '.join([
                '接收包数' ,
                f'增长[{Statistics_Currently.Receive.Packets[X]}]%' ,
                f'趋势[{Statistics_Currently.Receive.Packets[X] - Statistics_Previous.Receive.Packets[X]}]%' ,
                f'本次[{Performance_Currently.Receive.Packets[X]}]个' ,
                f'上次[{Performance_Previous.Receive.Packets[X]}]个' ,
            ]));
            Source.append(' | '.join([
                '入丢包次数' ,
                f'增长[{Statistics_Currently.Drop.In[X]}]%' ,
                f'趋势[{Statistics_Currently.Drop.In[X] - Statistics_Previous.Drop.In[X]}]%' ,
                f'本次[{Performance_Currently.Drop.In[X]}]次' ,
                f'上次[{Performance_Previous.Drop.In[X]}]次' ,
            ]));
            Source.append(' | '.join([
                '出丢包字节' ,
                f'增长[{Statistics_Currently.Drop.Out[X]}]%' ,
                f'趋势[{Statistics_Currently.Drop.Out[X] - Statistics_Previous.Drop.Out[X]}]%' ,
                f'本次[{Performance_Currently.Drop.Out[X]}]次' ,
                f'上次[{Performance_Previous.Drop.Out[X]}]次' ,
            ]));
            Source.append(' | '.join([
                '入错误次数' ,
                f'增长[{Statistics_Currently.Error.In[X]}]%' ,
                f'趋势[{Statistics_Currently.Error.In[X] - Statistics_Previous.Error.In[X]}]%' ,
                f'本次[{Performance_Currently.Error.In[X]}]次' ,
                f'上次[{Performance_Previous.Error.In[X]}]次' ,
            ]));
            Source.append(' | '.join([
                '出错误字节' ,
                f'增长[{Statistics_Currently.Error.Out[X]}]%' ,
                f'趋势[{Statistics_Currently.Error.Out[X] - Statistics_Previous.Error.Out[X]}]%' ,
                f'本次[{Performance_Currently.Error.Out[X]}]次' ,
                f'上次[{Performance_Previous.Error.Out[X]}]次' ,
            ]));
        return Source;
    
    @staticmethod
    def Disk() -> list :
        Statistics_Previous , Statistics_Currently = DURATION_STATISTICS_DISK;
        Performance_Previous , Performance_Currently = DURATION_PERFORMANCE_DISK;
        Source = list();
        Source.append('系统磁盘');
        Source.append(' | '.join([
            '读字节' ,
            f'增长[{Statistics_Currently.Read}]%' ,
            f'趋势[{Statistics_Currently.Read - Statistics_Previous.Read}]%' ,
            f'本次[{Performance_Currently.Read}]bytes' ,
            f'上次[{Performance_Previous.Read}]bytes' ,
        ]));
        Source.append(' | '.join([
            '写字节' ,
            f'增长[{Statistics_Currently.Write}]%' ,
            f'趋势[{Statistics_Currently.Write - Statistics_Previous.Write}]%' ,
            f'本次[{Performance_Currently.Write}]bytes' ,
            f'上次[{Performance_Previous.Write}]bytes' ,
        ]));
        Source.append(' | '.join([
            '忙碌时间' ,
            f'增长[{Statistics_Currently.Time}]ms' ,
            f'趋势[{Statistics_Currently.Time - Statistics_Previous.Time}]ms' ,
            f'本次[{Performance_Currently.Time}]ms' ,
            f'上次[{Performance_Previous.Time}]ms' ,
        ]));
        return Source;
    
    @staticmethod
    def File() -> list :
        Statistics_Previous , Statistics_Currently = DURATION_STATISTICS_FILE;
        Performance_Previous , Performance_Currently = DURATION_PERFORMANCE_FILE;
        Source = list();
        Source.append('系统文件');
        Source.append(' | '.join([
            '文件句柄' ,
            f'增长[{Statistics_Currently}]次' ,
            f'趋势[{Statistics_Currently - Statistics_Previous}]次' ,
            f'本次[{Performance_Currently}]次' ,
            f'上次[{Performance_Previous}]次' ,
        ]));
        return Source;
    
    @staticmethod
    def Load() -> list :
        Statistics_Previous , Statistics_Currently = DURATION_STATISTICS_LOAD;
        Performance_Previous , Performance_Currently = DURATION_PERFORMANCE_LOAD;
        Source = list();
        Source.append('系统负载');
        Source.append(' | '.join([
            '每1分钟' ,
            f'增长[{Statistics_Currently.One}]%' ,
            f'趋势[{Statistics_Currently.One - Statistics_Previous.One}]%' ,
            f'本次[{Performance_Currently.One}]%' ,
            f'上次[{Performance_Previous.One}]%' ,
        ]));
        Source.append(' | '.join([
            '每5分钟' ,
            f'增长[{Statistics_Currently.Five}]%' ,
            f'趋势[{Statistics_Currently.Five - Statistics_Previous.Five}]%' ,
            f'本次[{Performance_Currently.Five}]%' ,
            f'上次[{Performance_Previous.Five}]%' ,
        ]));
        Source.append(' | '.join([
            '每15分钟' ,
            f'增长[{Statistics_Currently.Fifteen}]%' ,
            f'趋势[{Statistics_Currently.Fifteen - Statistics_Previous.Fifteen}]%' ,
            f'本次[{Performance_Currently.Fifteen}]%' ,
            f'上次[{Performance_Previous.Fifteen}]%' ,
        ]));
        return Source;

class Statistics :
    
    @staticmethod
    def Cpu() -> Library.Simple :
        Previous , Currently = DURATION_PERFORMANCE_CPU;
        Source = Library.Simple();
        Source.Process = Currently.Process - Previous.Process;
        Source.Thread = Currently.Thread - Previous.Thread;
        Source.Core = Currently.Core - Previous.Core;
        Source.Respective = list();
        Source.Expense = dict();
        for Index , Value in enumerate(Currently.Respective) : Source.Respective.append(Value - Previous.Respective[Index]);
        for Pid in Currently.Expense :
            if Pid in Previous.Expense : Source.Expense[Pid] = Library.Simple(
                Cpu = Currently.Expense[Pid].Cpu - Previous.Expense[Pid].Cpu ,
                Memory = Currently.Expense[Pid].Memory - Previous.Expense[Pid].Memory ,
                Name = Currently.Expense[Pid].Name ,
            );
        return Source;
    
    @staticmethod
    def Memory() -> Library.Simple :
        Previous , Currently = DURATION_PERFORMANCE_MEMORY;
        Source = Library.Simple();
        Source.Total = Currently.Total - Previous.Total;
        Source.Used = Currently.Used - Previous.Used;
        Source.Available = Currently.Available - Previous.Available;
        Source.Cached = Currently.Cached - Previous.Cached;
        Source.Percent = Currently.Percent - Previous.Percent;
        return Source;
    
    @staticmethod
    def Network() -> Library.Simple :
        Previous , Currently = DURATION_PERFORMANCE_NETWORK;
        Source = Library.Simple(
            Send = Library.Simple(
                Bytes = dict() ,
                Packets = dict() ,
            ) ,
            Receive = Library.Simple(
                Bytes = dict() ,
                Packets = dict() ,
            ) ,
            Drop = Library.Simple(
                In = dict() ,
                Out = dict() ,
            ) ,
            Error = Library.Simple(
                In = dict() ,
                Out = dict() ,
            ) ,
        );
        for X in Currently.Send.Bytes : Source.Send.Bytes[X] = Currently.Send.Bytes[X] - Previous.Send.Bytes[X];
        for X in Currently.Send.Packets : Source.Send.Packets[X] = Currently.Send.Packets[X] - Previous.Send.Packets[X];
        for X in Currently.Receive.Bytes : Source.Receive.Bytes[X] = Currently.Receive.Bytes[X] - Previous.Receive.Bytes[X];
        for X in Currently.Receive.Packets : Source.Receive.Packets[X] = Currently.Receive.Packets[X] - Previous.Receive.Packets[X];
        for X in Currently.Drop.In : Source.Drop.In[X] = Currently.Drop.In[X] - Previous.Drop.In[X];
        for X in Currently.Drop.Out : Source.Drop.Out[X] = Currently.Drop.Out[X] - Previous.Drop.Out[X];
        for X in Currently.Error.In : Source.Error.In[X] = Currently.Error.In[X] - Previous.Error.In[X];
        for X in Currently.Error.Out : Source.Error.Out[X] = Currently.Error.Out[X] - Previous.Error.Out[X];
        return Source;
    
    @staticmethod
    def Disk() -> Library.Simple :
        Previous , Currently = DURATION_PERFORMANCE_DISK;
        Source = Library.Simple();
        Source.Read = Currently.Read - Previous.Read;
        Source.Write = Currently.Write - Previous.Write;
        Source.Time = Currently.Time - Previous.Time;
        return Source;
    
    @staticmethod
    def File() -> int :
        Previous , Currently = DURATION_PERFORMANCE_FILE;
        Source = Currently - Previous;
        return int(Source);
    
    @staticmethod
    def Load() -> Library.Simple :
        Previous , Currently = DURATION_PERFORMANCE_LOAD;
        Source = Library.Simple();
        Source.One = Currently.One - Previous.One;
        Source.Five = Currently.Five - Previous.Five;
        Source.Fifteen = Currently.Fifteen - Previous.Fifteen;
        return Source;

class Performance :
    
    @staticmethod
    def Cpu() -> Library.Simple :
        Source = Library.Simple(
            Process = 0 ,
            Thread = 0 ,
            Core = round(Library.Psutil.cpu_percent() , 10) ,
            Respective = Library.Psutil.cpu_percent(percpu = True) ,
            Expense = dict() ,
        );
        try :
            for X in Library.Psutil.process_iter() :
                try :
                    Source.Process += 1;
                    Source.Thread += X.num_threads();
                except : continue;
        except :
            Source.Process = -1;
            Source.Thread = -1;
        for X in Library.Psutil.Process(Library.Os.getppid()).children(recursive = True) : Source.Expense[X.pid] = Library.Simple(
            Cpu = round(X.cpu_percent() , 10) ,
            Memory = round(X.memory_info().rss / 1024 / 1024 , 10) ,
            Name = X.name() ,
        );
        return Source;
    
    @staticmethod
    def Memory() -> Library.Simple :
        Virtual = Library.Psutil.virtual_memory();
        return Library.Simple(
            Total = round(Virtual.total / 1024 / 1024 , 10) ,
            Used = round(Virtual.used / 1024 / 1024 , 10) ,
            Available = round(Virtual.available / 1024 / 1024 , 10) ,
            Cached = round(getattr(Virtual , 'cached' , 0) / 1024 / 1024 , 10) ,
            Percent = round(Virtual.percent , 10) ,
        );
    
    @staticmethod
    def Network() -> Library.Simple :
        Current = Library.Simple(
            Send = Library.Simple(
                Bytes = dict() ,
                Packets = dict() ,
            ) ,
            Receive = Library.Simple(
                Bytes = dict() ,
                Packets = dict() ,
            ) ,
            Drop = Library.Simple(
                In = dict() ,
                Out = dict() ,
            ) ,
            Error = Library.Simple(
                In = dict() ,
                Out = dict() ,
            ) ,
        );
        Counters = Library.Psutil.net_io_counters(pernic = True)
        for X in Counters :
            Current.Send.Bytes[X] = round(Counters[X].bytes_sent , 10);
            Current.Receive.Bytes[X] = round(Counters[X].bytes_recv , 10);
            Current.Send.Packets[X] = round(Counters[X].packets_sent , 10);
            Current.Receive.Packets[X] = round(Counters[X].packets_recv , 10);
            Current.Drop.In[X] = round(Counters[X].dropin , 10);
            Current.Drop.Out[X] = round(Counters[X].dropout , 10);
            Current.Error.In[X] = round(Counters[X].errin , 10);
            Current.Error.Out[X] = round(Counters[X].errout , 10);
        return Current;
    
    @staticmethod
    def Disk() -> Library.Simple :
        Counters = Library.Psutil.disk_io_counters();
        return Library.Simple(
            Read = round(Counters.read_bytes , 10) ,
            Write = round(Counters.write_bytes , 10) ,
            Time = getattr(Counters , 'busy_time' , 0) ,
        );
    
    @staticmethod
    def File() -> int :
        Process = Library.Psutil.Process();
        try : return Process.num_fds();
        except :
            try : return Process.num_handles();
            except : return -1;
    
    @staticmethod
    def Load() -> Library.Simple :
        Avg = Library.Os.getloadavg();
        Source = Library.Simple(
            One = round(Avg[0] , 10) ,
            Five = round(Avg[1] , 10) ,
            Fifteen = round(Avg[2] , 10) ,
        );
        return Source;